from lxml import html
from base64 import b64decode
from argparse import ArgumentParser
from furl import furl
from re import compile, match
from .http_client import request, BASE_URL

def extract_links_from_page(page):
    """ Extract links from a given page 
    
    Keyword arguments:
    page -- String containing body of an HTTP request made of a specific movie/show page

    """
    links = []
    tree = html.fromstring(page)
    anchors = tree.xpath('//span[@class="movie_version_link"]/a')

    for item in anchors:
        event_str = str(item.get('onclick'))
        href = str(item.get('href'))
        test = compile(r"^\S+url=\S+$")
        if isinstance(event_str, str) and match(test, href) and '_host' not in event_str:
            query = furl(BASE_URL + href)
            decoded_link = b64decode(query.args['url'])
            links.append(decoded_link)
    
    if len(links) > 0:
        return str(b'\n'.join(links).decode())
    else:
        raise SystemExit('primewire: error: No links were found')

def extract_page_from_results(results, options={}):
    """ Extract links page from search results 
    
    Keyword arguments:
    results -- String containing body of an HTTP request made from search results
    options -- Includes options for shows such as 'season' and 'episode' (default {})

    """
    tree = html.fromstring(results)
    anchors = tree.xpath('//div[@class="index_item index_item_ie"]/a')
    
    if len(anchors) > 0:
        link = anchors[0].get('href')
        contents = ''
        if 'season' in options and 'episode' in options:
            link = link.replace('watch', 'tv')
            season = options['season']
            episode = options['episode']
            if not isinstance(season, int) and not isinstance(episode, int):
                try:
                    season = int(season)
                    episode = int(episode)
                except ValueError as ve:
                    raise ve
            contents = request(BASE_URL + link + '/season-%d' % season + '-episode-%d' % episode)
        else:
            contents = request(BASE_URL + link)
        
        return extract_links_from_page(contents)
    else:
        raise SystemExit('primewire: error: Title not found')

def search(title, year=None, season=None, episode=None):
    """ Search for links 
    
    Keyword arguments:
    title -- title of show/movie
    year -- year of show/movie (default None)
    season -- season of show (default None)
    episode -- episode of show (default None)

    """
    title = title.strip().lower()
    section = 2 if isinstance(season, int) and isinstance(episode, int) else 1

    params = { 'search_keywords': title, 'search_section' : section }
    options = { 'season': season, 'episode': episode } if section == 2 else {}

    if isinstance(year, int):
        params['year'] = year
    
    return extract_page_from_results(request(BASE_URL, params), options)

def main():
    parser = ArgumentParser()
    parser.add_argument('title', help='title of the show or movie', nargs='+',type=str)
    parser.add_argument('-y', '--year', help='year of show or movie', type=int)
    parser.add_argument('-s', '--season', help='season selection', type=int)
    parser.add_argument('-e', '--episode', help='episode selection', type=int)
    args = parser.parse_args()

    print(search(' '.join(args.title), args.year, args.season, args.episode))
